-----------------------------------------PART 3A (32-bit ALU)--------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;

ENTITY alu IS
	PORT(
			a : IN STD_LOGIC_VECTOR(31 DOWNTO 0);			--1st operand									(32-bit)				
			b : IN STD_LOGIC_VECTOR(31 DOWNTO 0);			--2nd operand									(32-bit)
			op : IN STD_LOGIC_VECTOR(2 DOWNTO 0);			--Operation Selection (Control Line)	(2-bit)
			result : INOUT STD_LOGIC_VECTOR(31 DOWNTO 0);--Operation Result 							(32-bit)
			cout : OUT STD_LOGIC;								--Carry out flag								(1-bit)
			zero : OUT STD_LOGIC);								--Zero Flag										(1-bit)
END alu;

ARCHITECTURE description OF alu IS 
-----------------INTERNAL COMPONENTS-------------------	
	-- 32-bit Adder
	COMPONENT adder_32bit IS
		PORT (
			a_in 		: IN STD_LOGIC_VECTOR(31 DOWNTO 0);		--1st operand									
			b_in 		: IN STD_LOGIC_VECTOR(31 DOWNTO 0);		--2nd operand									
			c_in 		: IN STD_LOGIC;                  		--Carry in (maybe from prev. stage)	
			result 	: OUT STD_LOGIC_VECTOR(31 DOWNTO 0);	--Result 							
			c_out 	: OUT STD_LOGIC								--Carry out (for Program Status Register [PSR])
		);
	END COMPONENT;
	
	-- 32-bit AND Gate 
	COMPONENT and_32bit IS
		PORT(
			a_in 		: IN STD_LOGIC_VECTOR(31 downto 0);		--1st operand									
			b_in 		: IN STD_LOGIC_VECTOR(31 downto 0);		--2nd operand									
			result 	: OUT STD_LOGIC_VECTOR(31 downto 0)		--Result 		
		);			
	END COMPONENT;
	
	-- 32-bit OR Gate 
	COMPONENT or_32bit IS
		PORT(
			a_in 		: IN STD_LOGIC_VECTOR(31 downto 0);		--1st operand									
			b_in 		: IN STD_LOGIC_VECTOR(31 downto 0);		--2nd operand									
			result 	: OUT STD_LOGIC_VECTOR(31 downto 0)		--Result 		
		);			
	END COMPONENT;
	
	-- 32-bit Logical Shift Right
	COMPONENT right_shift_32bit IS
			PORT(
			a_in 		: IN STD_LOGIC_VECTOR(31 downto 0);		--1st operand																	
			result 	: OUT STD_LOGIC_VECTOR(31 downto 0)		--Result 			
		);			
	END COMPONENT;
	
	-- 32-bit Logical Shift Left
	COMPONENT left_shift_32bit IS
		PORT(
			a_in 		: IN STD_LOGIC_VECTOR(31 downto 0);		--1st operand																	
			result 	: OUT STD_LOGIC_VECTOR(31 downto 0)		--Result 			
		);			
	END COMPONENT;
	
	-- 32-bit 8-to-1 Multiplexer
	COMPONENT mux_8to1 IS
		PORT(
			in0,in1,in2,in3,in4,in5,in6,in7		: IN STD_LOGIC_VECTOR(31 downto 0);	--MUX signal input choices										
			sel 											: IN STD_LOGIC_VECTOR(2 downto 0);	--Select									
			y 												: OUT STD_LOGIC_VECTOR(31 downto 0)	--MUX output signal	
		);			
	END COMPONENT;
	
	-- 32-bit 2-to-1 Multiplexer
	COMPONENT mux_2to1 IS
		PORT(
			in0,in1		: IN STD_LOGIC_VECTOR(31 downto 0);	--MUX signal input choices										
			sel 			: IN STD_LOGIC;							--Select		(1-bit)							
			y 			   : OUT STD_LOGIC_VECTOR(31 downto 0)	--MUX output signal
		);			
	END COMPONENT;
	
	-- 32-bit NOT Gate
	COMPONENT not_32bit IS
		PORT(
			x 	: IN STD_LOGIC_VECTOR(31 downto 0);		--Input																	
			y 	: OUT STD_LOGIC_VECTOR(31 downto 0)		--Output	
		);			
	END COMPONENT;
	
-----------------INTERNAL WIRES-------------------		(connected to outputs of components, then to MUX)
signal andResult		:STD_LOGIC_VECTOR(31 downto 0);
signal orResult		:STD_LOGIC_VECTOR(31 downto 0);
signal adderResult	:STD_LOGIC_VECTOR(31 downto 0);
signal lslResult		:STD_LOGIC_VECTOR(31 downto 0);
signal lsrResult		:STD_LOGIC_VECTOR(31 downto 0);
signal bNegOrNot		:STD_LOGIC_VECTOR(31 downto 0);
signal bNot				:STD_LOGIC_VECTOR(31 downto 0);
	
BEGIN
	-- Connect 8-to-1 MUX
	mux1:			mux_8to1		PORT MAP(in0		=> andResult,		-- mapping object inputs to signals (based on lab manual)
												in1		=> orResult,
												in2		=> adderResult,
												in3		=> "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ", -- no signal set in manual
												in4		=> lslResult,
												in5		=> lsrResult,
												in6		=> adderResult,		-- subtraction result
												in7		=> "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ", -- no signal set in manual
												sel		=> op,
												y			=> result);

	-- Connect 32-bit adder
	adder1:     adder_32bit PORT MAP(a_in		=> a,
												b_in		=> bNegOrNot,
												c_in 		=> op(2),
												result	=> adderResult,
												c_out		=> cout);
	-- Connect 32-bit AND gate
	and1:			and_32bit	PORT MAP(a,b,andResult);
	-- Connect 32-bit OR gate
	or1:			or_32bit		PORT MAP(a,b,orResult);
	-- Connect 32-bit Logical Shift Right
	lsr1:			right_shift_32bit	PORT MAP(a,lsrResult);
	-- Connect 32-bit Logical Shift Left
	lsl1:			left_shift_32bit	PORT MAP(a,lslResult);
	-- Connect b negative decider MUX
	negMux:		mux_2to1		PORT MAP(in0		=> b,
												in1		=> bNot,			-- NOT(b) connect to 1 of MUX (bNot output of NOT gate)
												sel		=> op(2),
												y			=> bNegOrNot);
	-- Connect 32-bit NOT gate
	notber:		not_32bit	PORT MAP(b,bNot);			-- in not_32bit, b := x and bNot (wire) := y
	
	PROCESS (result)
	BEGIN
		IF result = "00000000000000000000000000000000" THEN
			zero <= '1';
		ELSE
			zero <= '0';
		END IF;
	END PROCESS;
END description;