library ieee;
use ieee.std_logic_1164.ALL;

ENTITY control IS
		PORT(
				clk, mclk : IN STD_LOGIC;
				enable : IN STD_LOGIC;
				statusC, statusZ : IN STD_LOGIC;
				INST : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
				A_Mux, B_Mux : OUT STD_LOGIC;
				IM_MUX1, REG_Mux : OUT STD_LOGIC;
				IM_MUX2, DATA_Mux : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
				ALU_op : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
				inc_PC, ld_PC : OUT STD_LOGIC;
				clr_IR : OUT STD_LOGIC;
				ld_IR : OUT STD_LOGIC;
				clr_A, clr_B, clr_C, clr_Z : OUT STD_LOGIC;
				ld_A, ld_B, ld_C, ld_Z : OUT STD_LOGIC;
				T : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
				wen, en : OUT STD_LOGIC
				);
		END control;
		
ARCHITECTURE description OF control IS
		TYPE STATETYPE IS (state_0, state_1, state_2);	-- define various states (T0, T1, T2)
		SIGNAL present_state, next_state: STATETYPE;		-- present_state signal of type STATETYPE (can be any signal state_0, state_1...)
		SIGNAL opCode: STD_LOGIC_VECTOR(3 downto 0);
		SIGNAL funcCode: STD_LOGIC_VECTOR(3 downto 0);
BEGIN

-------- OPERATION DECODER ---------
PROCESS (present_state, INST, statusC, statusZ, enable)
BEGIN
	CASE present_state IS
	
		WHEN state_0 =>
			clr_IR <= '0'; ld_IR <= '1'; 				-- For state_0, only thing is loading address into IR
			ld_PC <= '0'; inc_PC <= '0';
			ld_A <= '0'; clr_A <= '0';
			ld_B <= '0'; clr_B <= '0';
			ld_C <= '0'; clr_C <= '0';
			ld_Z <= '0'; clr_Z <= '0';
			ALU_op <= "000";								-- Make 000 where XXX
			A_Mux <= '0'; B_Mux <= '0';
			REG_Mux <= '0';
			DATA_Mux <= "00";								-- DATA_Mux must be 00 (from the memory to get IR value)
			IM_MUX1 <= '0'; IM_MUX2 <= "00";
		
		WHEN state_1 =>
			clr_IR <= '0'; ld_IR <= '0';
			ld_PC <= '1'; inc_PC <= '1';
			ld_A <= '0'; clr_A <= '0';
			ld_B <= '0'; clr_B <= '0';
			ld_C <= '0'; clr_C <= '0';
			ld_Z <= '0'; clr_Z <= '0';
			ALU_op <= "000";								
			A_Mux <= '0'; B_Mux <= '0';
			REG_Mux <= '0';
			DATA_Mux <= "00";
			IM_MUX1 <= '0'; IM_MUX2 <= "00";	
			
			IF (opCode = "0010") THEN				--STA
				REG_Mux <= '0';
				
			ELSIF (opCode = "0011") THEN			--STB
				REG_Mux <= '1';
			
			ELSIF (opCode = "0110") THEN			--BEQ
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "110";										--SUB A and B
				A_Mux <= '0'; B_Mux <= '0';				
				REG_Mux <= '0';							
				DATA_Mux <= "00";									
				IM_MUX1 <= '0'; IM_MUX2 <= "00";			
			
			ELSIF (opCode = "1000") THEN			--BNE
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "110";										--SUB A and B
				A_Mux <= '0'; B_Mux <= '0';				
				REG_Mux <= '0';							
				DATA_Mux <= "00";									
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
			
			END IF;
			
		WHEN state_2 =>
			IF (opCode = "1001") THEN		-- LDA
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '0'; B_Mux <= '0';		-- A_Mux = 0
				REG_Mux <= '0';
				DATA_Mux <= "01";						-- Output from Data Memory
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
				
			ELSIF (opCode = "1010") THEN		-- LDB
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '1'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '0'; B_Mux <= '0';		-- B_Mux = 0
				REG_Mux <= '0';
				DATA_Mux <= "01";						-- Output from Data Memory
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
			
			ELSIF (opCode = "0010") THEN		-- STA
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '0'; B_Mux <= '0';		
				REG_Mux <= '0';							-- Reg_Mux is 0
				DATA_Mux <= "00";						
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
				
			ELSIF (opCode = "0011") THEN		-- STB
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '0'; B_Mux <= '0';		
				REG_Mux <= '1';							-- Reg_Mux is 1
				DATA_Mux <= "00";						
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
			
			ELSIF (opCode = "0101") THEN		-- JMP
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '1'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '0'; B_Mux <= '0';		
				REG_Mux <= '0';							
				DATA_Mux <= "00";						
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
			
			ELSIF (opCode = "0000") THEN		-- LDAI
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '1'; B_Mux <= '0';				-- For IMM value, go through LZE
				REG_Mux <= '0';							
				DATA_Mux <= "00";						
				IM_MUX1 <= '0'; IM_MUX2 <= "00";		
				
			ELSIF (opCode = "0001") THEN		-- LDBI
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '1'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								
				A_Mux <= '0'; B_Mux <= '1';				-- For IMM value, go through LZE
				REG_Mux <= '0';							
				DATA_Mux <= "00";						
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
				
			ELSIF (opCode = "0100") THEN		-- LUI
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '1';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "001";								
				A_Mux <= '0'; B_Mux <= '0';		-- Value in Reg. A		
				REG_Mux <= '0';							
				DATA_Mux <= "10";						-- From ALU		
				IM_MUX1 <= '1'; IM_MUX2 <= "00";	-- Through UZE for upper value
				
			ELSIF (opCode = "0111" and funcCode = "1001") THEN		-- ANDI
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "000";								-- 000 for AND			
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "01";			-- Select IMM value from IR
				
			ELSIF (opCode = "0111" and funcCode = "1110") THEN		-- DECA
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "110";								-- 110 for SUB	
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "10";			-- Use constant 1 to decrement
				
			ELSIF (opCode = "0111" and funcCode = "0000") THEN		-- ADD
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "010";								-- 010 for ADD	
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "00";			-- Take B value
				
			ELSIF (opCode = "0111" and funcCode = "0010") THEN		-- SUB
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "110";								-- 110 for SUB	
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "00";			-- Take B value
				
			ELSIF (opCode = "0111" and funcCode = "0011") THEN		-- INCA
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "010";								-- 010 for ADD	
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "10";			-- Use constant 1 to increment
				
			ELSIF (opCode = "0111" and funcCode = "1011") THEN		-- AND
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "000";								-- 000 for AND
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "00";			-- Take B value
				
			ELSIF (opCode = "0111" and funcCode = "0001") THEN		-- ADDI
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "010";								-- 010 for ADD	
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "01";			-- Through LZE
				
			ELSIF (opCode = "0111" and funcCode = "1101") THEN		-- ORI
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "001";								-- 001 for OR
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "01";			-- Through LZE
				
			ELSIF (opCode = "0111" and funcCode = "0100") THEN		-- ROL
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "100";								-- 100 for ROL
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "00";			-- Only rolls one value (A)
				
			ELSIF (opCode = "0111" and funcCode = "1111") THEN		-- ROR
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '1'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '1'; clr_C <= '0';
				ld_Z <= '1'; clr_Z <= '0';
				ALU_op <= "101";								-- 101 for ROR
				A_Mux <= '0'; B_Mux <= '0';				-- A value
				REG_Mux <= '0';							
				DATA_Mux <= "10";								-- Select ALU output	
				IM_MUX1 <= '0'; IM_MUX2 <= "00";			-- Only rolls one value (A)
				
			ELSIF (opCode = "0111" and funcCode = "0101") THEN		-- CLRA
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '1';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								-- assign 0s for don't cares
				A_Mux <= '0'; B_Mux <= '0';				
				REG_Mux <= '0';							
				DATA_Mux <= "00";									
				IM_MUX1 <= '0'; IM_MUX2 <= "00";		
			
			ELSIF (opCode = "0111" and funcCode = "0110") THEN		-- CLRB
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '1';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								-- assign 0s for don't cares
				A_Mux <= '0'; B_Mux <= '0';				
				REG_Mux <= '0';							
				DATA_Mux <= "00";									
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
				
			ELSIF (opCode = "0111" and funcCode = "0111") THEN		-- CLRC
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '1';
				ld_Z <= '0'; clr_Z <= '0';
				ALU_op <= "000";								-- assign 0s for don't cares
				A_Mux <= '0'; B_Mux <= '0';				
				REG_Mux <= '0';							
				DATA_Mux <= "00";									
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
				
			ELSIF (opCode = "0111" and funcCode = "1000") THEN		-- CLRZ
				clr_IR <= '0'; ld_IR <= '0';
				ld_PC <= '0'; inc_PC <= '0';
				ld_A <= '0'; clr_A <= '0';
				ld_B <= '0'; clr_B <= '0';
				ld_C <= '0'; clr_C <= '0';
				ld_Z <= '0'; clr_Z <= '1';
				ALU_op <= "000";								-- assign 0s for don't cares
				A_Mux <= '0'; B_Mux <= '0';				
				REG_Mux <= '0';							
				DATA_Mux <= "00";									
				IM_MUX1 <= '0'; IM_MUX2 <= "00";	
				
			ELSIF (opCode = "0111" and funcCode = "1010") THEN		-- TSTZ
				IF (statusZ = '1') THEN 
					clr_IR <= '0'; ld_IR <= '0';
					ld_PC <= '1'; inc_PC <= '1';		-- increment PC
					ld_A <= '0'; clr_A <= '0';
					ld_B <= '0'; clr_B <= '0';
					ld_C <= '0'; clr_C <= '0';
					ld_Z <= '0'; clr_Z <= '0';
					ALU_op <= "000";								-- assign 0s for don't cares
					A_Mux <= '0'; B_Mux <= '0';				
					REG_Mux <= '0';							
					DATA_Mux <= "00";									
					IM_MUX1 <= '0'; IM_MUX2 <= "00";
				END IF;	
				
			ELSIF (opCode = "0111" and funcCode = "1100") THEN		-- TSTC
				IF (statusC = '1') THEN 
					clr_IR <= '0'; ld_IR <= '0';
					ld_PC <= '1'; inc_PC <= '1';		-- increment PC
					ld_A <= '0'; clr_A <= '0';
					ld_B <= '0'; clr_B <= '0';
					ld_C <= '0'; clr_C <= '0';
					ld_Z <= '0'; clr_Z <= '0';
					ALU_op <= "000";								-- assign 0s for don't cares
					A_Mux <= '0'; B_Mux <= '0';				
					REG_Mux <= '0';							
					DATA_Mux <= "00";									
					IM_MUX1 <= '0'; IM_MUX2 <= "00";
				END IF;	
				
			ELSIF (opCode = "0110") THEN		-- BEQ
				IF (statusZ = '1') THEN 						-- If Z flag high -> A-B=0 -> A=B
					clr_IR <= '0'; ld_IR <= '0';
					ld_PC <= '1'; inc_PC <= '0';		-- PC <= IR[15..0]
					ld_A <= '0'; clr_A <= '0';
					ld_B <= '0'; clr_B <= '0';
					ld_C <= '0'; clr_C <= '0';
					ld_Z <= '0'; clr_Z <= '0';
					ALU_op <= "000";								-- assign 0s for don't cares
					A_Mux <= '0'; B_Mux <= '0';				
					REG_Mux <= '0';							
					DATA_Mux <= "00";									
					IM_MUX1 <= '0'; IM_MUX2 <= "00";
				END IF;	
				
			ELSIF (opCode = "1000") THEN		-- BNE
				IF (statusZ = '0') THEN 						-- If Z flag low -> A-B != 0 -> A != B
					clr_IR <= '0'; ld_IR <= '0';
					ld_PC <= '1'; inc_PC <= '0';		-- PC <= IR[15..0]
					ld_A <= '0'; clr_A <= '0';
					ld_B <= '0'; clr_B <= '0';
					ld_C <= '0'; clr_C <= '0';
					ld_Z <= '0'; clr_Z <= '0';
					ALU_op <= "000";								-- assign 0s for don't cares
					A_Mux <= '0'; B_Mux <= '0';				
					REG_Mux <= '0';							
					DATA_Mux <= "00";									
					IM_MUX1 <= '0'; IM_MUX2 <= "00";
				END IF;	
	
			END IF;	
		END CASE;
END PROCESS;


-------- STATE MACHINE ---------
PROCESS (clk, enable)
BEGIN
	case enable is
		when '1' =>
			if(rising_edge(clk)) then
				case present_state is
					when state_2 => present_state <= state_0;
						T <= "001";
						opCode <= INST(31 downto 28);
						funcCode <= INST(27 downto 24);
					when state_1 => present_state <= state_2;
						T <= "100";
					when state_0 => present_state <= state_1;
						T <= "010";
				end case;
			end if;
		when '0' => present_state <= state_0;
			T <= "001";
	end case;
END PROCESS;


-------- DATA MEMORY INSTRUCTIONS ---------
PROCESS (mclk, clk, INST)
BEGIN
	IF(mclk'EVENT and mclk = '0') THEN
		IF(present_state = state_1 AND clk = '0') THEN
		
			--LDA and LDB Signals
			IF (opCode = "1001" or opCode = "1010") THEN
				en <= '1';
				wen <= '0';
				
			--STA and STB Signals
			ELSIF (opCode = "0010" or opCode = "0011") THEN
				en <= '1';
				wen <= '1';
				
			--Default Case Signals	
			ELSE 
				en <= '0';
				wen <= '0';
			END IF;
		ELSIF(present_state = state_2 AND clk = '1') THEN
			--LDA and LDB
			IF (opCode = "1001" or opCode = "1010") THEN
				en <= '0';
				wen <= '0';
				
			--STA and STB
			ELSIF (opCode = "0010" or opCode = "0011") THEN
				en <= '0';
				wen <= '0';
			--Default Case
			ELSE 
				en <= '0';
				wen <= '0';
			END IF;
		ELSIF(present_state = state_1) THEN --or alternatively just an ELSE statement
			en <= '0';
			wen <= '0';
		END IF;
	END IF;
END process;
END description;