LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY data_mem IS
PORT(
		clk 		: IN STD_LOGIC;							
		addr 		: IN UNSIGNED(7 DOWNTO 0);
		data_in 	: IN STD_LOGIC_VECTOR(31 DOWNTO 0);
		wen 		: IN STD_LOGIC;
		en 		: IN STD_LOGIC;
		data_out : OUT STD_LOGIC_VECTOR(31 DOWNTO 0));
END data_mem;

ARCHITECTURE Description OF data_mem IS

-- define array type
	TYPE mem_array IS ARRAY (255 DOWNTO 0) OF
		STD_LOGIC_VECTOR(31 DOWNTO 0);

-- define signal to use array
	SIGNAL mem	: mem_array;
	
-- Address signal
	SIGNAL address	: INTEGER RANGE 0 TO 255;  
	
	BEGIN
	PROCESS (addr, data_in, wen, clk, en)
		BEGIN
		address <= TO_INTEGER(addr);	--address should be an integer type
		IF en = '0' THEN
			data_out <= "00000000000000000000000000000000";
		ELSIF clk'event AND clk = '0' THEN			--falling edge triggered
			IF (wen = '1') THEN
				mem(address) <= data_in;	--write data (data_in) to memory
			ELSIF (wen = '0') THEN
				data_out <= mem(address);	--data read value from memory
			ELSE
				data_out <= "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ"; --data_out is highest condition
			END IF;
		END IF;
	END PROCESS;
END Description;