LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;

ENTITY reset_circuit IS
	PORT
	(
		Reset : IN STD_LOGIC;
		Clk : IN STD_LOGIC;
		Enable_PD : OUT STD_LOGIC;
		Clr_PC : OUT STD_LOGIC
	);
END reset_circuit;

ARCHITECTURE description OF reset_circuit IS 

SIGNAL count : integer := 0;		-- initialize counter to 0    

BEGIN
	PROCESS (Clk, Reset, count)				-- For Finite State Machine
	BEGIN
		
		IF(Clk'event AND Clk = '1') THEN
			CASE Reset IS
			
			WHEN '1' =>					-- When reset activated
				Enable_PD <= '0';
				Clr_PC <= '1';			-- Clear PC
				count <= 1;				-- (1st cycle of count) clears PC 
			
			WHEN '0' =>					-- When reset not activated
				IF (count = 0) THEN
					Enable_PD <= '1'; 
					Clr_PC <= '0';
				END IF;
				
				IF (count >= 1 and count <= 3) THEN		-- If cycle between 1st and 3rd (T0 to T2)
					count <= count + 1;						-- increments counter (hold values)
				END IF;
				
				IF (count = 4) THEN							-- If count is 4 (done stabilizing)
					Enable_PD <= '1';							-- Enable active (regular without reset)
					Clr_PC <= '0';
				END IF;
			END CASE;
		END IF;
		
	END PROCESS;
END description;




